package sun.misc;

import java.util.Hashtable;

public final class Signal
{
    private static Hashtable<Signal, SignalHandler> handlers;
    private static Hashtable<Integer, Signal> signals;
    private int number;
    private String name;
    
    public int getNumber() {
        return this.number;
    }
    
    public String getName() {
        return this.name;
    }
    
    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (o == null || !(o instanceof Signal)) {
            return false;
        }
        Signal signal = (Signal)o;
        return this.name.equals(signal.name) && this.number == signal.number;
    }
    
    @Override
    public int hashCode() {
        return this.number;
    }
    
    @Override
    public String toString() {
        return "SIG" + this.name;
    }
    
    public Signal(String s) {
        this.number = findSignal(s);
        this.name = s;
        if (this.number < 0) {
            throw new IllegalArgumentException("Unknown signal: " + s);
        }
        return;
    }
    
    public static synchronized SignalHandler handle(Signal obj, SignalHandler value) throws IllegalArgumentException {
        long n = (value instanceof NativeSignalHandler) ? ((NativeSignalHandler)value).getHandler() : 2L;
        long handle0 = handle0(obj.number, n);
        if (handle0 == -1L) {
            throw new IllegalArgumentException("Signal already used by VM or OS: " + obj);
        }
        Signal.signals.put(Integer.valueOf(obj.number), obj);
        synchronized (Signal.handlers) {
            SignalHandler signalHandler = (SignalHandler)Signal.handlers.get(obj);
            Signal.handlers.remove(obj);
            if (n == 2L) {
                Signal.handlers.put(obj, value);
            }
            if (handle0 == 0L) {
                return SignalHandler.SIG_DFL;
            }
            if (handle0 == 1L) {
                return SignalHandler.SIG_IGN;
            }
            if (handle0 == 2L) {
                return signalHandler;
            }
            return new NativeSignalHandler(handle0);
        }
    }
    
    public static void raise(Signal signal) throws IllegalArgumentException {
        if (Signal.handlers.get(signal) == null) {
            throw new IllegalArgumentException("Unhandled signal: " + signal);
        }
        raise0(signal.number);
        return;
    }
    
    private static void dispatch(int i) {
        final Signal signal = (Signal)Signal.signals.get(Integer.valueOf(i));
        final SignalHandler signalHandler = (SignalHandler)Signal.handlers.get(signal);
        Runnable task = new Runnable() {
            @Override
            public void run() {
                signalHandler.handle(signal);
            }
        };
        if (signalHandler != null) {
            new Thread(task, signal + " handler").start();
        }
    }
    
    private static native int findSignal(String p0);
    
    private static native long handle0(int p0, long p1);
    
    private static native void raise0(int p0);
    
    static {
        Signal.handlers = new Hashtable(4);
        Signal.signals = new Hashtable(4);
    }
}
